/// @addtogroup covrel
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file graphtset.cpp
///
/// @author Jaroslaw Dulak, Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include "covrel/graphtset.h"

namespace capd{
namespace covrel{

typedef GraphicsTripleSet::FloatVector FloatVector;
typedef GraphicsTripleSet::IntervalVector IntervalVector;


void GraphicsTripleSet::show(Frame &f) const
{
   // plotting in Yellow - the N^- edges - but for better visualition
	FloatVector S = center + unstable - 2.*stable;
	FloatVector E = center + unstable + 2.*stable;
	f.line(S[0],S[1],E[0],E[1],YELLOW);

	S = center - unstable - 2.*stable;
	E = center - unstable + 2.*stable;
	f.line(S[0],S[1],E[0],E[1],YELLOW);

   // plotting of the support
	f.jump(A[0],A[1]);
	f.draw(B[0],B[1]);
	f.draw(C[0],C[1]);
	f.draw(D[0],D[1]);
	f.draw(A[0],A[1]);

   // plotting the stable direction in BLUE
	f.jump(center[0],center[1]);
	f.draw(center[0] + stable[0],center[1] + stable[1],BLUE);

   // plotting the unstable direction in RED
	f.jump(center[0],center[1]);
	f.draw(center[0] + unstable[0],center[1] + unstable[1],RED);
}

// -----------------------------------------------------------------

void GraphicsTripleSet::show(Frame &f, const FloatVector& point, int color) const
{
	show(f);													//we plot support
	f.Xcrss(point[0],point[1],5,color);  //we plot the point
}

// -----------------------------------------------------------------

void GraphicsTripleSet::show(Frame &f, const IntervalVector& vect, int color) const
{
	show(f);
	f.boxFill(vect[0].leftBound(),vect[1].leftBound(),vect[0].rightBound(),vect[1].rightBound(),color);
	f.box(vect[0].leftBound(),vect[1].leftBound(),vect[0].rightBound(),vect[1].rightBound(),BLACK);
}

// -----------------------------------------------------------------

Frame& operator<<(Frame& f, GraphicsTripleSet& Set)
{
   Set.show(f);
	return f;
}

}} // namespace capd::covrel

/// @}
