#include "symcov.h"

//###########################################################//

void clearFrame(Frame &f, double SIZE, frstring title)
{
   double minx = -SIZE;
   double maxx = SIZE;
   double miny = -SIZE;
   double maxy = SIZE;
   f.Clear();
   f.setWorldCoord(minx,miny,maxx,maxy);
   f.Bound(GREEN);
   f.jump(minx,0.);
   f.draw(maxx,0.);
   f.jump(0.,miny);
   f.draw(0.,maxy);
   f << title;
}

//###########################################################//

void clearFrames(double SIZE1, double SIZE2)
{
   clearFrame(graph1,SIZE1,"unstable direcitions");
   clearFrame(graph2,SIZE2,"stable directions");
   txt.Bound(BROWN);
}

//###########################################################//

void initGraph()
{
   openGW(1000,710,WHITE,BLACK);
   graph1 = Frame(0,0,500,500,WHITE,BLACK);
   graph2 = Frame(500,0,1000,500,WHITE,BLACK);
   txt = Frame(100,510,900,700);
}

//###########################################################//

void readFixedPoints(vector &P1, vector &P2)
{
   std::ifstream file;
   file.open("fixedpoints");
   file >> P1 >> P2;
   file.close();
}

//###########################################################//

void readEigenvectors(vector eV1[], vector eV2[])
{
   std::ifstream file;

   file.open("eigenvectors1");
   file >> eV1[0];
   file >> eV1[1];
   file.close();

// eV1[2] = ReversingSymmetry(eV[0])
   eV1[2][0] = -eV1[0][0];
   eV1[2][1] = -eV1[0][1];
   eV1[2][2] =  eV1[0][2];
   eV1[2][3] =  eV1[0][3];

// eV1[3] = ReversingSymmetry(eV[1])
   eV1[3][0] = -eV1[1][0];
   eV1[3][1] = -eV1[1][1];
   eV1[3][2] =  eV1[1][2];
   eV1[3][3] =  eV1[1][3];


   file.open("eigenvectors2");
   file >> eV2[0];
   file >> eV2[1];
   file.close();

// eV2[2] = ReversingSymmetry(eV2[0])
   eV2[2][0] = -eV2[0][0];
   eV2[2][1] = -eV2[0][1];
   eV2[2][2] =  eV2[0][2];
   eV2[2][3] =  eV2[0][3];

// eV2[3] = ReversingSymmetry(eV2[1])
   eV2[3][0] = -eV2[1][0];
   eV2[3][1] = -eV2[1][1];
   eV2[3][2] =  eV2[1][2];
   eV2[3][3] =  eV2[1][3];
}

//###########################################################//

void initMatricesOfEigenvectors(matrix &A1, matrix &A2, vector eV1[], vector eV2[])
{
   int i,j;
   for(i=0;i<4;i++)
   {
      for(j=0;j<4;j++)
      {
         A1[j][i] = eV1[i][j];
         A2[j][i] = eV2[i][j];
      }
   }
}

//###########################################################//

void initLinearApproximations()
{
   std::ifstream file;
   matrix temp;

   file.open("linear");
   file >> temp;
   linearApproxN1 = intervalMatrix(temp);
   file >> temp;
   linearApproxH1 = intervalMatrix(temp);
   file >> temp;
   linearApproxH2 = intervalMatrix(temp);
   file >> temp;
   linearApproxH3 = intervalMatrix(temp);
   file >> temp;
   linearApproxN2 = intervalMatrix(temp);
   file.close();
}

//###########################################################//

void initSets()
{
   vector eigenVectorsP1[4], eigenVectorsP2[4];
   matrix eigenSystemP1, eigenSystemP2;
   vector fixedPointP1, Q1, Q2, Q3, fixedPointP2;
   vector sizeN1, sizeH1, sizeH2, sizeH3, sizeH4, sizeN2;

   readFixedPoints(fixedPointP1, fixedPointP2);
   readEigenvectors(eigenVectorsP1, eigenVectorsP2);
   initMatricesOfEigenvectors(eigenSystemP1, eigenSystemP2, eigenVectorsP1, eigenVectorsP2);
   initLinearApproximations();

   Q1 = fixedPointP1 + 0.0330092432*eigenVectorsP1[0] - 0.048949*eigenVectorsP1[1] + 0.0004931*eigenVectorsP1[2];
   Q2 = example(Q1,4);
   Q3 = example(Q2,1);

   sizeN1[0] = .012; sizeN1[1] = .012;   sizeN1[2] = .012; sizeN1[3] = .012;
   sizeH1[0] = .001; sizeH1[1] = .00175; sizeH1[2] = .005; sizeH1[3] = .005;
   sizeH2[0] = .28;  sizeH2[1] = .28;    sizeH2[2] = .2;  sizeH2[3] = .38;
   sizeH3[0] = .15;  sizeH3[1] = .15;    sizeH3[2] = .12;  sizeH3[3] = .42;
   sizeN2[0] = .31;   sizeN2[1] = .31;     sizeN2[2] = .31;   sizeN2[3] = .31;

   N1 = new symSet(eigenSystemP1, fixedPointP1, sizeN1);
   H1 = new symSet(eigenSystemP1, Q1, sizeH1);
   H2 = new symSet(eigenSystemP2, Q2, sizeH2);
   H3 = new symSet(eigenSystemP2, Q3, sizeH3);
   N2 = new symSet(eigenSystemP2, fixedPointP2, sizeN2);
}

//###########################################################//

void deleteSets()
{
   delete N1;
   delete H1;
   delete H2;
   delete H3;
   delete N2;
}
