#include "symcov.h"


//###########################################################//

interval g1(const interval &x1,const interval &x2)
{
   return x1*(interval(1.)-x1)+interval(4.)-x2;
}

//###########################################################//

interval g2(const interval &x1,const interval &x2)
{
   return x2*(interval(1.)-x2)+interval(4.)+x1;
}

//###########################################################//

intervalVector example(const intervalVector &v)
{
   intervalVector result;
   interval G1 = g1(v[0]+v[2],v[1]+v[3])/interval(2.);
   interval G2 = g2(v[0]+v[2],v[1]+v[3])/interval(2.);
   result[0] = -v[2] + G1;
   result[1] = -v[3] + G2;
   result[2] =  v[0] + G1;
   result[3] =  v[1] + G2;

   return result;
}

//###########################################################//

intervalVector example(intervalVector x, int ile)
{
   numberOfBoxes++;
   int i;
   for(i=0;i<ile;i++)
      x = example(x);
   return x;
}

//###########################################################//

void plot(intervalVector &v, bool change, symSet *dest, int color)
{
   if(GRAPH)
      dest->show(graph1,graph2,v,color,change);
}

//###########################################################//

int computeUnstableWall(symSet *source, intervalVector v, int iterations,
               int grid2, int grid3, int skip, symSet *dest,
               intervalVector &linearImage)
{
   interval skip2 = source->Isize[2]/interval(grid2);
   interval skip3 = source->Isize[3]/interval(grid3);
   interval I(0.,1.);
   interval box2 = skip2 * I;
   interval box3 = skip3 * I;
   int i,j;

   for(i=-grid2;i<grid2;i+=skip)
   {
      for(j=-grid3;j<grid3;j+=skip)
      {
         v[2] = interval(i)*skip2 + box2;
         v[3] = interval(j)*skip3 + box3;
         intervalVector w = source->IM*(v);
         w += source->Icenter;
         w = example(w,iterations);
         w = gauss(dest->IM,w-(dest->Icenter));
         linearImage = intervalHull(linearImage,w);
         if(!dest->outUnstable(linearImage,false))
         {
            if(GRAPH)
            {
               plot(linearImage,false,dest,BLUE);
               rootFrame << "inclusion not satified!";
               waitBt();
            }
            writeReport("Inclusion not satisfied!\n");
            frstring f;
            f << "v[2]=" << i <<", v[3]=" << j;
            writeReport(f.string());
            return 0;
         }
      }
   }

   plot(linearImage,false,dest,RED);
   return 1;
}

//###########################################################//

void computeImageOfUnstableWalls(symSet *source, int iterations,
      int grid0, int grid1, int grid2, int grid3,  int skip, symSet *dest,
      intervalVector (*linear)(const intervalVector &),
      int factor1, int factor2)
{
   if(GRAPH)
   {
      double SIZE1 = factor1*(dest->size[0]> dest->size[1] ? dest->size[0] : dest->size[1]);
      double SIZE2 = factor2*(dest->size[2]> dest->size[3] ? dest->size[2] : dest->size[3]);
      clearFrames(SIZE1,SIZE2);
   }

   interval skip0 = source->Isize[0]/interval(grid0);
   interval skip1 = source->Isize[1]/interval(grid1);
   interval I(0.,1.);
   interval box0 = skip0 * I;
   interval box1 = skip1 * I;
   int j;
   intervalVector v;

   if(GRAPH) dest->show(graph1,graph2);

   v[0] = -(source->Isize[0]);
   for(j=-grid1;j<grid1;j+=skip)
   {
      v[1] = j*skip1 + box1;
      v[2] = v[3] = interval(0.);
      intervalVector linearImage = linear(v);
      linearImage = gauss(dest->IM,linearImage - dest->Icenter);
// projection onto unstable directions
      linearImage[2] = linearImage[3] = 0.;
      if(!computeUnstableWall(source,v,iterations,grid2,grid3,skip,dest,linearImage))
      {
         frstring f;
         f << "\nv[0]=negative v[1]=" << j;
         writeReport(f.string());
         deleteSets();
         exit(0);
      }
   }

   v[0] = source->Isize[0];
   for(j=-grid1;j<grid1;j+=skip)
   {
      v[1] = j*skip1 + box1;
      v[2] = v[3] = interval(0.);
      intervalVector linearImage = linear(v);
      linearImage = gauss(dest->IM,linearImage - dest->Icenter);
// projection onto unstable directions
      linearImage[2] = linearImage[3] = 0.;
      if(!computeUnstableWall(source,v,iterations,grid2,grid3,skip,dest,linearImage))
      {
         frstring f;
         f << "\nv[0]=positive v[1]=" << j;
         writeReport(f.string());
         deleteSets();
         exit(0);
      }
   }
   v[1] = -(source->Isize[1]);
   for(j=-grid0;j<grid0;j+=skip)
   {
      v[0] = j*skip0 + box0;
      v[2] = v[3] = interval(0.);
      intervalVector linearImage = linear(v);
      linearImage = gauss(dest->IM,linearImage - dest->Icenter);
// projection onto unstable directions
      linearImage[2] = linearImage[3] = 0.;
      if(!computeUnstableWall(source,v,iterations,grid2,grid3,skip,dest,linearImage))
      {
         frstring f;
         f << "\nv[1]=negative v[0]=" << j;
         writeReport(f.string());
         deleteSets();
         exit(0);
      }
   }

   v[1] = source->Isize[1];
   for(j=-grid0;j<grid0;j+=skip)
   {
      v[0] = j*skip0 + box0;
      v[2] = v[3] = interval(0.);
      intervalVector linearImage = linear(v);
      linearImage = gauss(dest->IM,linearImage - dest->Icenter);
// projection onto unstable directions
      linearImage[2] = linearImage[3] = 0.;
      if(!computeUnstableWall(source,v,iterations,grid2,grid3,skip,dest,linearImage))
      {
         frstring f;
         f << "\nv[1]=positive v[0]=" << j;
         writeReport(f.string());
         deleteSets();
         exit(0);
      }
   }

   if(GRAPH) dest->show(graph1,graph2);
}

//###########################################################//

int computeStableWall(symSet *source, intervalVector v, int iterations,
               int grid0, int grid1, int skip, symSet *dest)
{
   interval skip0 = source->Isize[0]/interval(grid0);
   interval skip1 = source->Isize[1]/interval(grid1);
   interval I(0.,1.);
   interval box0 = skip0 * I;
   interval box1 = skip1 * I;
   int i,j;

// first approximation of the bonund
   v[0] = v[1] = interval(0.);
   intervalVector bound = source->IM*(v);
   bound += source->Icenter;
   bound = example(bound,iterations);
   bound = gauss(dest->IM,bound-(dest->Icenter));

   for(i=-grid0;i<grid0;i+=skip)
   {
      for(j=-grid1;j<grid1;j+=skip)
      {
         v[0] = interval(i)*skip0 + box0;
         v[1] = interval(j)*skip1 + box1;
         intervalVector w = source->IM*(v);
         w += source->Icenter;
         w = example(w,iterations);
         w = gauss(dest->IM,w-(dest->Icenter));
         bound = intervalHull(bound,w);
         if(!dest->across(bound,false))
         {
            if(GRAPH)
            {
               plot(bound,false,dest,BLUE);
               rootFrame << "inclusion not satified!";
               waitBt();
            }

            writeReport("Inclusion not satisfeid!\n");
            frstring f;
            f << "v[0]=" << i <<", v[1]=" << j;
            writeReport(f.string());
            return 0;
         }
      }
   }

   if(GRAPH) plot(bound,false,dest,RED);
   return 1;
}

//###########################################################//

void computeImageOfStableWalls(symSet *source, int iterations,
      int grid0, int grid1, int grid2, int grid3,  int skip, symSet *dest,
      int factor1, int factor2)
{
   if(GRAPH)
   {
      double SIZE1 = factor1*(dest->size[0]> dest->size[1] ? dest->size[0] : dest->size[1]);
      double SIZE2 = factor2*(dest->size[2]> dest->size[3] ? dest->size[2] : dest->size[3]);
      clearFrames(SIZE1,SIZE2);
   }

   interval skip2 = source->Isize[2]/interval(grid2);
   interval skip3 = source->Isize[3]/interval(grid3);
   interval I(0.,1.);
   interval box2 = skip2 * I;
   interval box3 = skip3 * I;
   int j;

   intervalVector v;
   if(GRAPH) dest->show(graph1,graph2);
   v[2] = -(source->Isize[2]);
   for(j=-grid3;j<grid3;j+=skip)
   {
      v[3] = j*skip3 + box3;
      if(!computeStableWall(source,v,iterations,grid0,grid1,skip,dest))
      {
         frstring f;
         f << "v[2]=negative, v[3]=" << j;
         writeReport(f.string());
         deleteSets();
         exit(0);
      }
   }

   v[2] = source->Isize[2];
   for(j=-grid3;j<grid3;j+=skip)
   {
      v[3] = j*skip3 + box3;
      if(!computeStableWall(source,v,iterations,grid0,grid1,skip,dest))
      {
         frstring f;
         f << "v[2]=positive, v[3]=" << j;
         writeReport(f.string());
         deleteSets();
         exit(0);
      }
   }

   v[3] = -(source->Isize[3]);
   for(j=-grid2;j<grid2;j+=skip)
   {
      v[2] = j*skip2 + box2;
      if(!computeStableWall(source,v,iterations,grid0,grid1,skip,dest))
      {
         frstring f;
         f << "v[3]=negative, v[2]=" << j;
         writeReport(f.string());
         deleteSets();
         exit(0);
      }
   }

   v[3] = source->Isize[3];
   for(j=-grid2;j<grid2;j+=skip)
   {
      v[2] = j*skip2 + box2;
      if(!computeStableWall(source,v,iterations,grid0,grid1,skip,dest))
      {
         frstring f;
         f << "v[3]=positive, v[2]=" << j;
         writeReport(f.string());
         deleteSets();
         exit(0);
      }
   }

   if(GRAPH) dest->show(graph1,graph2);
}
