#include "symcov.h"

//###########################################################//

double g1(double x1,double x2)
{
   return x1*(1-x1)+4-x2;
}

//###########################################################//

double g2(double x1,double x2)
{
   return x2*(1-x2)+4+x1;
}

//###########################################################//

vector example(const vector &v)
{
   vector result;
   double G1 = g1(v[0]+v[2],v[1]+v[3]);
   double G2 = g2(v[0]+v[2],v[1]+v[3]);
   result[0] = -v[2] + 0.5*G1;
   result[1] = -v[3] + 0.5*G2;
   result[2] =  v[0] + 0.5*G1;
   result[3] =  v[1] + 0.5*G2;
   return result;
}

//###########################################################//

vector example(vector x, int ile)
{
   int i;
   for(i=0;i<ile;i++)
      x = example(x);
   return x;
}

//###########################################################//

void plot(vector &v, bool change, symSet *dest)
{
   dest->show(graph1,graph2,v,BLUE,change);
}

//###########################################################//

void showWall(symSet *source, vector v, int iterations, bool change, vector &grid, int skip, symSet *dest)
{
   double skip2 = skip*grid[2];
   double skip3 = skip*grid[3];

   for(v[2]= -(source->size[2]);v[2]<=(source->size[2]); v[2]+=skip2)
   for(v[3]= -(source->size[3]);v[3]<=(source->size[3]); v[3]+=skip3)
   {
      vector w = source->M*v;
      w += source->center;
      w = example(w,iterations);
      plot(w,change,dest);
   }
}

//###########################################################//

void showImageOfUnstableWalls(symSet *source, int iterations, bool change, int Grid, int skip, symSet *dest,
                  int factor1, int factor2)
{
   double SIZE1 = factor1*(dest->size[0]> dest->size[1] ? dest->size[0] : dest->size[1]);
   double SIZE2 = factor2*(dest->size[2]> dest->size[3] ? dest->size[2] : dest->size[3]);
   clearFrames(SIZE1,SIZE2);

   vector v;
   vector grid = (source->size)/static_cast<double>(Grid);
   double L,R;

   L = -(source->size[1]);
   R = source->size[1];
   v[0] = -(source->size[0]);
   for(v[1]=L; v[1]<=R; v[1]+=grid[1])
      showWall(source,v,iterations,change,grid,skip,dest);

   v[0] = source->size[0];
   for(v[1]=L; v[1]<=R; v[1]+=grid[1])
      showWall(source,v,iterations,change,grid,skip,dest);

   L = -(source->size[0]);
   R = source->size[0];
   v[1] =-( source->size[1]);
   for(v[0]=L; v[0]<=R;v[0]+=grid[0])
      showWall(source,v,iterations,change,grid,skip,dest);

   v[1] = source->size[1];
   for(v[0]=L; v[0]<=R; v[0]+=grid[0])
      showWall(source,v,iterations,change,grid,skip,dest);

   if(change)
   {
      dest->show(graph1,graph2);
   }
}

//###########################################################//

void showImageOfSet(symSet *source, int iterations, bool change, int Grid, int skip, symSet *dest,
                  int factor1, int factor2)
{
   double SIZE1 = factor1*(dest->size[0]> dest->size[1] ? dest->size[0] : dest->size[1]);
   double SIZE2 = factor2*(dest->size[2]> dest->size[3] ? dest->size[2] : dest->size[3]);
   clearFrames(SIZE1,SIZE2);

   vector v;
   vector grid = (source->size)/static_cast<double>(Grid);
   double L0 = -(source->size[0]);
   double L1 = -(source->size[1]);
   double R0 = source->size[0];
   double R1 = source->size[1];

   for(v[0]=L0; v[0]<=R0; v[0]+=grid[0])
      for(v[1]=L1; v[1]<=R1; v[1]+=grid[1])
         showWall(source,v,iterations,change,grid,skip,dest);

   if(change)
   {
      dest->show(graph1,graph2);
   }
}

//###########################################################//

void showImagesOfAllSets(int grid)
{
   if(GRAPH)
   {
      txt.Clear();
      txt << "The image of the set N1 in relation N1 cover N1\n";
      showImageOfSet(N1, 1, true, grid, 1, N1,10,2);
      waitBt();

      txt.Clear();
      txt << "The image of the set N1 in relation N1 cover H1\n";
      showImageOfSet(N1, 1, true, grid, 1, H1,80,2);
      waitBt();

      txt.Clear();
      txt << "The image of the set H1 in relation H1 cover H2\n";
      showImageOfSet(H1, 4, true, grid, 1, H2,10,2);
      waitBt();

      txt.Clear();
      txt << "The image of the set H2 in relation H2 cover H3\n";
      showImageOfSet(H2, 1, true, grid, 1, H3,10,2);
      waitBt();

      txt.Clear();
      txt << "The image of the set H3 in relation H3 cover N2\n";
      showImageOfSet(H3, 1, true, grid, 1, N2,10,2);
      waitBt();

      txt.Clear();
      txt << "The image of the set N2 in relation N2 cover N2\n";
      showImageOfSet(N2, 1, true, grid, 1, N2,10,2);
      waitBt();
   }
}

//###########################################################//

void showImagesOfAllUnstableWalls(int grid)
{
   if(GRAPH)
   {
      txt.Clear();
      txt << "The image of the unstable walls of N1 in relation N1 cover N1\n";
      showImageOfUnstableWalls(N1, 1, true, grid, 1, N1,10,2);
      waitBt();

      txt.Clear();
      txt << "The image of the unstable walls of N1 in relation N1 cover H1\n";
      showImageOfUnstableWalls(N1, 1, true, grid, 1, H1,80,2);
      waitBt();

      txt.Clear();
      txt << "The image of the unstable walls of H1 in relation H1 cover H2\n";
      showImageOfUnstableWalls(H1, 4, true, grid, 1, H2,10,2);
      waitBt();

      txt.Clear();
      txt << "The image of the unstable walls of H2 in relation H2 cover H3\n";
      showImageOfUnstableWalls(H2, 1, true, grid, 1, H3,15,2);
      waitBt();

      txt.Clear();
      txt << "The image of the unstable walls of H3 in relation H3 cover N2\n";
      showImageOfUnstableWalls(H3, 1, true, grid, 1, N2,10,2);
      waitBt();

      txt.Clear();
      txt << "The image of the unstable walls of N2 in relation N2 cover N2\n";
      showImageOfUnstableWalls(N2, 1, true, grid, 1, N2,10,2);
      waitBt();
   }
}
