#ifndef _SYMCOV_H_
#define _SYMCOV_H_

#include <fstream>
#include <cmath>

#include "krak/krak.h"
#include "interval/doubleInterval.h"
#include "vectalg/vector.h"
#include "vectalg/matrix.h"

typedef Vector<double,4> vector;
typedef Vector<interval,4> intervalVector;
typedef Matrix<double,4,4> matrix;
typedef Matrix<interval,4,4> intervalMatrix;

class symSet{
public:
   matrix M;
   vector center;
   vector size;

   intervalMatrix IM;
   intervalVector Icenter;
   intervalVector Isize;

   void verify() const;

public:
   symSet(matrix &_M, vector &_center, vector _size)
      : M(_M), center(_center), size(_size),
         IM(_M), Icenter(center), Isize(size)
      {
         verify();
      }
   symSet(intervalMatrix &_M, intervalVector &_center, intervalVector _size)
      : IM(_M), Icenter(_center), Isize(_size)
      {
         int i,j;
         for(i=0;i<M.numberOfRows();i++)
         {
            for(j=0;j<M.numberOfColumns();j++)
            {
               M[i][j] = IM[i][j].mid().leftBound();
            }
            center[i] = Icenter[i].mid().leftBound();
            size[i] = Isize[i].mid().leftBound();
         }
      }

   void show(Frame &f1, Frame &f2) const;
   void show(Frame &f1, Frame &f2, const vector &v, int color=BLACK, bool change=true) const;
   void show(Frame &f1, Frame &f2, const intervalVector &v, int color=BLACK, bool change=false) const;

   bool inSide(const intervalVector &v, bool change) const;
   bool outSide(const intervalVector &v, bool change) const;
   bool across(const intervalVector &v, bool change) const;
   bool outUnstable(const intervalVector &v, bool change) const;
}; // the end of class symSet

extern bool GRAPH;
extern Frame graph1, graph2, txt;
extern symSet *N1, *N2, *H1, *H2, *H3;
extern intervalMatrix linearApproxN1, linearApproxN2, linearApproxH1, linearApproxH2, linearApproxH3;
extern double numberOfBoxes;

// in file init.cpp
void initGraph();
void clearFrames(double,double);
void initSets();
void deleteSets();

// in file show.cpp
vector example(vector ,int);
void showImagesOfAllSets(int grid);
void showImagesOfAllUnstableWalls(int grid);

// in file rigorous.cpp
void computeImageOfUnstableWalls(symSet *source, int iterations,
      int grid0, int grid1, int grid2, int grid3,  int skip, symSet *dest,
            intervalVector (*f)(const intervalVector &), int, int);

void computeImageOfStableWalls(symSet *source, int iterations,
      int grid0, int grid1, int grid2, int grid3,  int skip, symSet *dest,int, int );

// in file covering.cpp
void cover();

// in file symmetric.cpp
void writeReport(const char *);

#endif
