#include "symcov.h"

void symSet::verify() const
{
   intervalVector A;
   gauss(IM,A);
}

void symSet::show(Frame &f1, Frame &f2) const
{
   f1.box(-size[0],-size[1],size[0],size[1]);
   f2.box(-size[2],-size[3],size[2],size[3]);
}

void symSet::show(Frame &f1, Frame &f2, const vector &v, int color, bool change) const
{
   if(change)
   {
      vector w=gauss(M,v-center);
      f1.dot(w[0],w[1],color);
      f2.dot(w[2],w[3],color);
   } else{
      f1.dot(v[0],v[1],color);
      f2.dot(v[2],v[3],color);
   }
}

void symSet::show(Frame &f1, Frame &f2, const intervalVector &v, int color, bool change) const
{
   if(change)
   {
      intervalVector w=gauss(IM,v-Icenter);
      f1.boxFill(w[0].leftBound(),w[1].leftBound(),w[0].rightBound(),w[1].rightBound(),color);
      f2.boxFill(w[2].leftBound(),w[3].leftBound(),w[2].rightBound(),w[3].rightBound(),color);
      f1.box(w[0].leftBound(),w[1].leftBound(),w[0].rightBound(),w[1].rightBound(),BLACK);
      f2.box(w[2].leftBound(),w[3].leftBound(),w[2].rightBound(),w[3].rightBound(),BLACK);
   } else{
      f1.boxFill(v[0].leftBound(),v[1].leftBound(),v[0].rightBound(),v[1].rightBound(),color);
      f2.boxFill(v[2].leftBound(),v[3].leftBound(),v[2].rightBound(),v[3].rightBound(),color);
      f1.box(v[0].leftBound(),v[1].leftBound(),v[0].rightBound(),v[1].rightBound(),BLACK);
      f2.box(v[2].leftBound(),v[3].leftBound(),v[2].rightBound(),v[3].rightBound(),BLACK);
   }
}

bool symSet::inSide(const intervalVector &v, bool change) const
{
   if(change)
   {
      intervalVector w = gauss(IM, v-Icenter);
      return (capd::abs(w[0]) < Isize[0]) &&
             (capd::abs(w[1]) < Isize[1]) &&
             (capd::abs(w[2]) < Isize[2]) &&
             (capd::abs(w[3]) < Isize[3]);
   } else
   {
      return (capd::abs(v[0]) < Isize[0]) &&
             (capd::abs(v[1]) < Isize[1]) &&
             (capd::abs(v[2]) < Isize[2]) &&
             (capd::abs(v[3]) < Isize[3]);
   }
}

bool symSet::outSide(const intervalVector &v, bool change) const
{
   if(change)
   {
      intervalVector w = gauss(IM, v-Icenter);
      return (capd::abs(w[0]) > Isize[0]) ||
             (capd::abs(w[1]) > Isize[1]) ||
             (capd::abs(w[2]) > Isize[2]) ||
             (capd::abs(w[3]) > Isize[3]);
   } else
   {
      return (capd::abs(v[0]) > Isize[0]) ||
             (capd::abs(v[1]) > Isize[1]) ||
             (capd::abs(v[2]) > Isize[2]) ||
             (capd::abs(v[3]) > Isize[3]);
   }
}

bool symSet::across(const intervalVector &v, bool change) const
{
   if(change)
   {
      intervalVector w = gauss(IM, v-Icenter);
      return ( (capd::abs(w[2]) < Isize[2]) && (capd::abs(w[3]) < Isize[3]) );
   } else
   {
      return ( (capd::abs(v[2]) < Isize[2]) && (capd::abs(v[3]) < Isize[3]) );
   }
}

bool symSet::outUnstable(const intervalVector &v, bool change) const
{
   return across(v,change) && outSide(v, change);
}
